#!/usr/bin/python -u
# coding: utf-8

#--------------------------------------------------------------------
# Fichero:  2-genera-nuc.py
# Objetivo: Generar un archivo csv con nombre, usuario y contraseña
# Fecha:    D.10.11.2019
# Autor:    Pedro Reina
# Licencia: Dominio público
#           https://creativecommons.org/publicdomain/zero/1.0/deed.es
#--------------------------------------------------------------------

#------------------------------------------------
# Variables del programa
#------------------------------------------------

# Carácter para comentarios en los archivos de datos
Comentario = '#'

# Archivo con los nombres. Codificación UTF-8
Origen = 'maxifeli-nombres.txt'

# Archivo con nombre, usuario y contraseña. Codificación UTF-8
Destino = 'maxifeli-nuc.csv'

# Caracteres para generar los códigos
Validos = 'abcdefghjmnpqrstuyz23456789'

# Longitud de los códigos
Longitud = 8

#------------------------------------------------
# Módulos necesarios
#------------------------------------------------

# Para escribir en Unicode en un archivo
import codecs

# Para generar números aleatorios
import random

# Para pasar a ASCII
import unicodedata

#------------------------------------------------
# Variables globales
#------------------------------------------------

# La lista con los nombres
ListaNombres = []

#------------------------------------------------
# Funciones
#------------------------------------------------

#------------------------------------------------
# Devuelve una cadena sin espacios por los lados
def CadenaLimpia(Cadena):
    return Cadena.strip()

#------------------------------------------------
# Devuelve una cadena convertida a ASCII
# y en minúsculas
def CadenaAsciiMinus(Cadena):
    Cadena = Cadena.lower()
    Cadena = unicodedata.normalize('NFKD', Cadena).encode('ascii','ignore')
    return Cadena

#------------------------------------------------
def GeneraCodigo():
    # Seleccionamos al azar el número de caracteres determinado
    Seleccion = random.sample(Validos, Longitud)

    # Creamos una cadena con los caracteres seleccionados
    Codigo = ''.join(Seleccion)

    # Ese es el código que hemos generado
    return Codigo

#------------------------------------------------
# Genera el archivo final
def GeneraDocumento():

  # Preparamos el fichero destino
  Salida = codecs.open (Destino, 'w', encoding='UTF-8')
  Salida.write ('Nombre,Usuario,Contrasena\n')

  # Para cada nombre
  for Nombre in ListaNombres:
      # Preparamos todos sus datos
      DatoNombre = Nombre.split(' ')
      Usuario = CadenaAsciiMinus(DatoNombre[0])
      Codigo = GeneraCodigo()

      # Escribimos en el archivo final
      Salida.write (Nombre + ',' + Usuario + ',' + Codigo + '\n')

  # Cerramos el archivo de texto
  Salida.close()

#------------------------------------------------
# Lee un archivo UTF-8 con comentarios y líneas en blanco
# y devuelve una lista con las líneas útiles limpias
def LeeDatos(Origen):
    ListaDatos = []

    Entrada = codecs.open(Origen, encoding='utf-8', mode='r')
    for Linea in Entrada:
        Linea = CadenaLimpia(Linea)
        if len(Linea) and Linea[0]!=Comentario:
            ListaDatos.append (Linea)

    return ListaDatos

#------------------------------------------------
# El programa
#------------------------------------------------

# Informamos al usuario
print '2-genera-nuc'
print '============'
print

# Leemos los datos
print 'Archivo origen:', Origen
print 'Leyendo origen...',
ListaNombres = LeeDatos(Origen)
print 'hecho.'

# Generamos el archivo
print 'Archivo destino:', Destino
print 'Generando destino...',
GeneraDocumento()
print 'hecho.'

# Hemos acabado; informamos al usuario
print 'Terminado.'
