#!/usr/bin/python -u
# coding: utf-8

#--------------------------------------------------------------------
# Fichero:  4-inventa-preferencias.py
# Objetivo: Insertar en la base de datos algunas preferencias
# Fecha:    D.10.11.2019
# Autor:    Pedro Reina
# Licencia: Dominio público
#           https://creativecommons.org/publicdomain/zero/1.0/deed.es
#--------------------------------------------------------------------

#------------------------------------------------
# Variables del programa
#------------------------------------------------

# El archivo con la base de datos
BaseDato = 'maxifeli.db'

#------------------------------------------------
# Módulos necesarios
#------------------------------------------------

# Another Python SQLite Wrapper
import apsw

# Para generar números aleatorios
import random

#------------------------------------------------
# Variables globales
#------------------------------------------------

# La lista con los usuarios
ListaUsuario = []

#------------------------------------------------
# Funciones
#------------------------------------------------

#------------------------------------------------
# Devuelve la lista de usuarios de la base de datos
def LeeUsuarios(Conexion):
    Lista = []
    Consulta = 'SELECT usuario FROM acceso'

    Cursor = Conexion.cursor()
    for Dato in Cursor.execute(Consulta):
        Lista.append(Dato[0])

    return Lista

#------------------------------------------------
def InsertaPreferencia(Conexion, Activo, Pasivo, Cantidad):
    Consulta = 'INSERT INTO preferencia VALUES (:Activo, :Pasivo, :Cantidad)'
    Cursor = Conexion.cursor()
    Cursor.execute(Consulta, locals())

#------------------------------------------------
def InsertaInventadas(Conexion):
    # Para cada usuario de la base de datos
    for Activo in ListaUsuario:
        # Elegimos seis candidatos, por si hay que desechar uno
        ListaPasivo = random.sample(ListaUsuario, 6)
        # Un usuario no puede emparejarse consigo mismo
        if Activo in ListaPasivo:
            ListaPasivo.remove(Activo)
        # Añadimos cinco preferencias
        for Cantidad in range(1,6):
            Pasivo = ListaPasivo[Cantidad-1]
            InsertaPreferencia(Conexion, Activo, Pasivo, Cantidad)

#------------------------------------------------
# El programa
#------------------------------------------------

# Informamos al usuario
print '4-inventa-preferencias'
print '======================'
print

# Leemos los datos
print 'Base de datos:', BaseDato
print 'Leyendo usuarios...',
# Abrimos conexión a la base de datos
Conexion = apsw.Connection(BaseDato)
# Averiguamos la lista de todos los usuarios
ListaUsuario = LeeUsuarios(Conexion)
print 'hecho.'

# Insertamos las preferencias inventadas
print 'Insertando preferencias...',
# Abrimos conexión a la base de datos
Conexion = apsw.Connection(BaseDato)
InsertaInventadas(Conexion)
print 'hecho.'

# Cerramos la conexión
Conexion.close(True)

# Hemos acabado; informamos al usuario
print 'Terminado.'
