<?php 

//--------------------------------------------------------------
// Fichero:  maxifeli/index.php 
// Objetivo: Página principal de MaxiFeli
// Autor:    Pedro Reina
// Fecha:    V.8.11.2019
// Licencia: Dominio público
//           https://creativecommons.org/publicdomain/zero/1.0/deed.es
//--------------------------------------------------------------

//------------------------------------------------
// Ficheros necesarios
//------------------------------------------------

require_once "maxifeli.php";

//------------------------------------------------
// Variable para ajustarse a cada caso
//------------------------------------------------

$NombreGrupo = '';

//------------------------------------------------
// Arrancamos el sistema de sesiones de PHP
//------------------------------------------------

session_start();

//------------------------------------------------
// Primero manejamos headers y luego la página
//------------------------------------------------

ManejaHeaders();
ManejaPagina();

//------------------------------------------------
// Funciones
//------------------------------------------------

//--------------------------------------------------------------
// Manejo de las cabeceras que debe enviar el servidor
function ManejaHeaders()
  {
  // Comprobamos si nos piden terminar
  if ( isset ($_POST['salir']) )
    {
    // Terminamos la sesión PHP
    $_SESSION = array();
    session_destroy();

    // Dirigimos el navegador a la página inicial
    header ('location: ' . $_SERVER['PHP_SELF']);
    }
  }

//--------------------------------------------------------------
// Desarrollo normal de la página
function ManejaPagina()
  {
  global $NombreGrupo;
  $Texto = 'Gestión de preferencias';
  if ( strlen($NombreGrupo) )
    { $Texto .= $NombreGrupo; }

  MF_DTD ("HTML401strict");
  print "<HTML>\n";
  MF_Cabecera ($Texto);
  print "<BODY>\n";

  // En la parte de arriba indicamos el propósito del sistema
  print "<DIV CLASS=\"cuadro\">\n";
  MF_Titulo ($Texto);
  print "</DIV>\n";

  // Miramos si el usuario se ha identificado
  if ( isset ($_SESSION['DentroMaxiFeli']) )
    { $Dentro = $_SESSION['DentroMaxiFeli']; }
  else
    { $Dentro = FALSE; }

  // Si no se ha identificado, le pedimos que lo haga
  if ( ! $Dentro )
    {
    // Si acabamos de cargar la página
    if ( ! isset ($_POST['codigo']) )
      { 
      // Pedimos al usuario que se identifique
      PideIdentificacion();
      }

    // Si ya han escrito la identificación, la comprobamos
    else
      {
      $Usuario = $_POST['usuario'];
      $Codigo = $_POST['codigo'];

      // Si la identificación es válida, dejamos pasar
      if ( MF_IdentificacionValida ($Usuario, $Codigo) )
        {
        // Anotamos que hemos entrado
        $Dentro = TRUE;
        $_SESSION['DentroMaxiFeli'] = TRUE;

        // Nos guardamos el usuario en la sesión
        $_SESSION['usuario'] = $Usuario;
        }
      else { IdentificacionIncorrecta(); }
      }
    }

  // Si se ha identificado
  if ( $Dentro )
    {
    $Usuario = $_SESSION['usuario'];
    EscribeNombreMenu ($Usuario);
    
    // Nos pueden pedir cambiar la contraseña
    if ( isset ($_POST['camcod']) )
      { CambiarCodigo(); }

    // Nos pueden mandar los datos para cambiar la contraseña
    else if ( isset ($_POST['codnuevo1']) )
      { EjecutarCambiarCodigo(); }

    // Nos pueden pedir que grabemos las nuevas preferencias
    else if ( isset ($_POST['grabar']) )
      { GrabaPreferencias ($Usuario); }

    // Si no nos piden nada de lo anterior, presentamos las preferencias
    else
      { EscribePreferencias ($Usuario); }
    }

  print "</BODY>\n";
  print "</HTML>\n";
  }

//--------------------------------------------------------------
// Pide usuario y contraseña para entrar al sistema
function PideIdentificacion()
  {
  $YoMismo = $_SERVER['PHP_SELF'];

  print "<DIV CLASS=\"cuadro\">";
  MF_Titulo2 ("Identificación");

  print "<P><FORM ACTION=\"$YoMismo\" METHOD=\"POST\">\n";
  print "<TABLE>\n";

  print "<TR>\n";
  print "<TD>Usuario:</TD>\n";
  print "<TD><INPUT CLASS=\"dato\" NAME=\"usuario\" " .
        "SIZE=\"8\" AUTOFOCUS></TD>\n";
  print "</TR>\n";

  print "<TR>\n";
  print "<TD>Contraseña:</TD>\n";
  print "<TD>\n";
  print "<INPUT TYPE=\"password\" CLASS=\"dato\" NAME=\"codigo\" SIZE=\"8\">\n";
  print "</TD>\n";
  print "</TR>\n";
  print "</TABLE>\n";

  print "<P CLASS=\"centrado\">\n";
  print "<INPUT TYPE=\"submit\" CLASS=\"orden entrar\" NAME=\"Entrar\" ";
  print "VALUE=\"Entrar\">\n";

  print "</FORM></P>\n";
  print "</DIV>\n";
  }

//--------------------------------------------------------------
// Invita a volver a identificarse tras un fallo
function IdentificacionIncorrecta()
  {
  $YoMismo = $_SERVER['PHP_SELF'];

  print "<DIV CLASS=\"cuadro\">";
  MF_Titulo2 ("Identificación");

  print "<P CLASS=\"centrado\">\n";
  print "Tu identificación no es válida</P>\n";

  print "<P CLASS=\"centrado separado\">\n";
  print "<A HREF=\"$YoMismo\" CLASS=\"orden reintentar\">Reintentar</A></P>";

  print "</DIV>\n";
  }

//--------------------------------------------------------------
// Escribe el nombre del usuario identificado y le ofrece el menú
function EscribeNombreMenu ($Usuario)
  {
  $YoMismo = $_SERVER['PHP_SELF'];

  print "<DIV CLASS=\"cuadro\">\n";
  MF_Titulo2 ("Nombre y menú");

  $Nombre = MF_Nombre($Usuario);

  print "<P><FORM ACTION=\"$YoMismo\" METHOD=\"POST\">\n";
  print "<SPAN STYLE=\"font-size: 24px; margin-right:20px;\">$Nombre</SPAN>\n";

  EscribeBoton ("camcod", "Contraseña", "camcod", "Cambiar la contraseña");
  EscribeBoton ("salir", "Salir", "salir", "Cierra la sesión");

  print "</FORM></P>\n";
  print "</DIV>\n";
  }

//--------------------------------------------------------------
// Escribe las preferencias de un usuario
function EscribePreferencias ($Usuario)
  {
  $YoMismo = $_SERVER['PHP_SELF'];

  print "<DIV CLASS=\"cuadro\">\n";
  MF_Titulo2 ("Preferencias");

  print "<P>Asigna a cada nombre la preferencia que desees.\n";
  print "Números más altos indican mayor predilección por\n";
  print "sentarte con esa persona.\n";

  $NombreDe = MF_ListaNombreDeUsuario($Usuario);
  $PreferenciaCon = MF_ListaPreferenciaConUsuario($Usuario);

  print "<P><FORM ACTION=\"$YoMismo\" METHOD=\"POST\">\n";

  print "<TABLE CLASS=\"listado\">\n";
  print "<TR>\n";
  print "<TD><B>Nombre</B></TD>\n";
  print "<TD><B>Preferencia</B></TD>\n";
  print "</TR>\n";

  foreach ($NombreDe as $u => $n)
    {
    // La preferencia de $Usuario con $u
    if ( array_key_exists($u, $PreferenciaCon) )
        { $Cantidad = $PreferenciaCon[$u]; }
    else
        { $Cantidad = 0; }

    print "<TR>\n";
    print "<TD>$n</TD>\n";
    print "<TD CLASS=\"centro\">\n";
    print "<SELECT NAME=\"$u\">\n";
    print "<OPTION VALUE=\"0\"></OPTION>\n";
    for ($i=1 ; $i<6 ; $i++ )
      {
      if ( $Cantidad == $i )
        { print "<OPTION VALUE=\"$i\" SELECTED>$i</OPTION>\n"; }
      else
        { print "<OPTION VALUE=\"$i\">$i</OPTION>\n"; }
      }
    print "</SELECT></TD>\n";
    print "</TR>\n";
    }

  print "</TABLE>\n";
  print "<P>\n";

  EscribeBoton ("grabar", "Grabar", "grabar", "Grabar los cambios");
  EscribeBoton ("anular", "Anular", "anular", "Descartar los cambios");

  print "</FORM></P>\n";
  print "</DIV>\n";
  }

//--------------------------------------------------------------
// Graba en la base de datos las preferencias de un usuario
function GrabaPreferencias ($Usuario)
  {
  print "<DIV CLASS=\"cuadro\">\n";
  MF_Titulo2 ("Preferencias");

  $NombreDe = MF_ListaNombreDeUsuario($Usuario);
  $PreferenciaCon = array();

  foreach ($NombreDe as $u => $n)
    {
    // La preferencia de $Usuario con $u
    $Cantidad = $_POST[$u];
    if ( $Cantidad )
      { $PreferenciaCon[$u] = $Cantidad; }
    }

  if ( $Error = MF_GrabaPreferencias ($Usuario, $PreferenciaCon) )
    { print (MF_Error ($Error)); }
  else
    { print (MF_Exito ("las nuevas preferencias se han grabado correctamente")); }

  print "<FORM><P>\n";
  EscribeBoton ("seguir", "Seguir", "seguir", "Continuar con el programa");
  print "</FORM></P>\n";

  print "</DIV>\n";
  }

//--------------------------------------------------------------
// Escribe un botón de orden
function EscribeBoton ($Orden, $Texto, $Nombre, $Indicacion)
  {
  print "<INPUT TYPE=\"submit\" CLASS=\"orden $Orden\" ";
  print "TITLE=\"$Indicacion\" NAME=\"$Nombre\" VALUE=\"$Texto\">\n";
  }

//--------------------------------------------------------------
// Permite cambiar su contraseña a un usuario
function CambiarCodigo()
  {
  $YoMismo = $_SERVER['PHP_SELF'];

  print "<DIV CLASS=\"cuadro\">";
  MF_Titulo2 ("Cambiar la contraseña");

  print "<P><FORM ACTION=\"$YoMismo\" METHOD=\"POST\">\n";
  print "<TABLE>\n";

  print "<TR>\n";
  print "<TD>Escribe la contraseña actual:</TD>\n";
  print "<TD>\n";
  print "<INPUT TYPE=\"password\" CLASS=\"dato\" NAME=\"codactual\" ";
  print "SIZE=\"8\">\n";
  print "</TD>\n";
  print "</TR>\n";

  print "<TR>\n";
  print "<TD>Escribe la nueva contraseña:</TD>\n";
  print "<TD>\n";
  print "<INPUT TYPE=\"password\" CLASS=\"dato\" NAME=\"codnuevo1\" ";
  print "SIZE=\"8\">\n";
  print "</TD>\n";
  print "</TR>\n";

  print "<TR>\n";
  print "<TD>Repite la nueva contraseña:</TD>\n";
  print "<TD>\n";
  print "<INPUT TYPE=\"password\" CLASS=\"dato\" NAME=\"codnuevo2\" ";
  print "SIZE=\"8\">\n";
  print "</TD>\n";
  print "</TR>\n";
  print "</TABLE>\n";

  print "<P CLASS=\"centrado\">\n";
  print "<INPUT TYPE=\"submit\" CLASS=\"orden cambiar\" NAME=\"cambiar\" ";
  print "VALUE=\"Cambiar\">\n";
  print "<A HREF=\"$YoMismo\" CLASS=\"orden anular\">Anular</A></P>\n";

  print "</FORM></P>\n";
  print "</DIV>\n";
  }

//--------------------------------------------------------------
// Cambia la contraseña de un usuario, tras comprobarlo todo
function EjecutarCambiarCodigo()
  {
  $YoMismo = $_SERVER['PHP_SELF'];

  print "<DIV CLASS=\"cuadro\">";
  MF_Titulo2 ("Cambiar la contraseña");

  // Recibimos los datos del formulario
  $CodigoActual = $_POST['codactual'];
  $CodigoNuevo1 = $_POST['codnuevo1'];
  $CodigoNuevo2 = $_POST['codnuevo2'];

  // Comprobamos la contraseña actual
  $Usuario = $_SESSION['usuario'];
  if ( ! MF_IdentificacionValida ($Usuario, $CodigoActual) )
    { print (MF_Error ("la contraseña actual no es correcta")); }

  // Comprobamos que las nuevas son iguales
  else if ( $CodigoNuevo1 != $CodigoNuevo2 )
    { print (MF_Error ("las contraseñas nuevas no coinciden")); }

  // Comprobamos que la nueva tiene longitud correcta
  else if ( $Error = MF_CompruebaCodigo ($CodigoNuevo1) )
    { print ($Error); }

  // Grabamos la nueva contraseña
  else
    {
    if ( $Error = MF_CambiaCodigo ($Usuario, $CodigoNuevo1) )
      { print (MF_Error ($Error)); }
    else
      { print (MF_Exito ("la contraseña se ha modificado correctamente")); }
    }

  print "<FORM><P>\n";
  EscribeBoton ("seguir", "Seguir", "seguir", "Continuar con el programa");
  print "</FORM></P>\n";
  }

?>
