<?php

//--------------------------------------------------------------
// Fichero:  maxifeli.php
// Objetivo: Rutinas PHP de apoyo a MaxiFeli
// Autor:    Pedro Reina
// Fecha:    V.8.11.2019
// Licencia: Dominio público
//           https://creativecommons.org/publicdomain/zero/1.0/deed.es
//--------------------------------------------------------------

//----------------------------
// Archivos necesarios
//----------------------------

// Manejo de bases de datos
require_once "basedato.php";

// Hash de cadenas
require_once "password.php";

//----------------------------
// Constantes
//----------------------------

// Características de las contraseñas
define ("MF_MIN_PASSWORD", 8);
define ("MF_MIN_PASSWORD_TEXTO", "ocho");

//----------------------------
// Variables globales
//----------------------------

//----------------------------
// Funciones que ofrecemos
//----------------------------

//--------------------------------------------------------------
// Emite el DTD de la página
function MF_DTD ($Descripcion)
  {
  if ( strcmp ($Descripcion, 'HTML401strict') == 0 )
    {
    print ("<!-- HTML 4.01 estricto -->\n");
    print ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\n");
    print ("  \"http://www.w3.org/TR/html4/strict.dtd\">\n");
    }
  }

//--------------------------------------------------------------
// La cabecera de la página
function MF_Cabecera ($Titulo)
  {
  $Icono = "imagen/maxifeli.png";
  $Hoja = "maxifeli.css";

  print "<HEAD>\n";
  print ("<TITLE>$Titulo</TITLE>\n");

  // Caracteres UTF-8
  print ("<META HTTP-EQUIV=\"Content-Type\" " .
         "CONTENT=\"text/html; charset=UTF-8\">\n");

  // Los robots no pueden indexar ni seguir enlaces
  print ("<META NAME=\"ROBOTS\" CONTENT=\"NOINDEX, NOFOLLOW\">\n");

  // El favicon
  print ("<LINK REL=\"icon\" HREF=\"$Icono\" " .
         "TYPE=\"image/png\">\n");

  // La hoja de estilos
  print ("<LINK REL=\"StyleSheet\" HREF=\"$Hoja\" TYPE=\"text/css\">\n");

  // Si además del título nos pasan más parámetros
  if ( func_num_args() > 1 )
    {
    // Examinamos cada argumento
    for ($i = 1 ; $i < func_num_args() ; $i++)
      {
      $Argumento = func_get_arg($i);

      // Comprobamos que no esté vacío
      if ( $Argumento != "" )
        {
        // Averiguamos la extensión
        preg_match ("#^.*\.([a-zA-Z0-9]+)#", $Argumento, $Encontrado);
        $Ext = $Encontrado[1];

        // Si la extensión es css, es una hoja de estilos
        if ( $Ext == "css" )
          {
          print ("<LINK REL=\"StyleSheet\" HREF=\"$Argumento\" ");
          print ("TYPE=\"text/css\">\n");
          }

        // Si la extensión es js, es un JavaScript
        else if ( $Ext == "js" )
          {
          print ("<SCRIPT TYPE=\"text/javascript\" SRC=\"$Argumento\">");
          print ("</SCRIPT>\n");
          }

        // Si no es ninguna de esas cosas, la enviamos tal cual
        else
          { print $Argumento . "\n"; }
        }
      }
    }

  print "</HEAD>\n";
  }

//--------------------------------------------------------------
// Muestra el título de una sección
function MF_Titulo ($Texto)
  { print ("<DIV CLASS=\"titulo1\">$Texto</DIV>\n"); }

//--------------------------------------------------------------
// Muestra el título de una sección de nivel dos
function MF_Titulo2 ($Texto)
  { print ("<DIV CLASS=\"titulo2\">$Texto</DIV>\n"); }

//--------------------------------------------------------------
// Devuelve el nombre de un usuario
function MF_Nombre ($Usuario)
  {
  $Consulta = "SELECT nombre FROM acceso WHERE usuario='$Usuario'";
  $Nombre =  BD_ConsultaValor ($Consulta);
  return $Nombre;
  }

//--------------------------------------------------------------
// Devuelve un array con los nombres de cada usuario diferente al dado
function MF_ListaNombreDeUsuario ($Usuario)
  {
  $Consulta = "SELECT usuario, nombre FROM acceso";
  $Lista =  BD_ConsultaListaRegistro ($Consulta);
  $Respuesta = array();
  
  foreach ($Lista as $Fila)
    {
    $u = $Fila['usuario'];
    $n = $Fila['nombre'];
    if ( $u != $Usuario )
      { $Respuesta[$u] = $n; }
    }

  return $Respuesta;
  }

//--------------------------------------------------------------
// Devuelve un array con las preferencias de un usaurio
function MF_ListaPreferenciaConUsuario ($Usuario)
  {
  $Consulta = "SELECT pasivo,cantidad FROM preferencia WHERE activo='$Usuario'";
  $Lista =  BD_ConsultaListaRegistro ($Consulta);
  $Respuesta = array();
  
  foreach ($Lista as $Fila)
    {
    $p = $Fila['pasivo'];
    $c = $Fila['cantidad'];
    $Respuesta[$p] = $c;
    }

  return $Respuesta;
  }

//--------------------------------------------------------------
// Graba en la base de datos un array con las preferencias de un usaurio
function MF_GrabaPreferencias ($Usuario, $PreferenciaCon)
  {
  // Borramos todas las preferencias anteriores del usuario
  $Consulta = "DELETE FROM preferencia WHERE activo='$Usuario'";
  $Error = BD_EjecutaConsulta ($Consulta);

  if ( !$Error )
    {
    // Preparamos las nuevas preferencias
    $ListaInsercion = array();
    foreach ( $PreferenciaCon as $u => $c )
      { $ListaInsercion[] = array ($u, $c); }

    // Insertamos las nuevas preferencias
    $Consulta = "INSERT INTO preferencia VALUES ('$Usuario', ?, ?)";
    $Error = BD_EjecutaConsultaLista ($Consulta, $ListaInsercion);
    }

  return $Error;
  }

//--------------------------------------------------------------
// Dice si un usuario corresponde con un código
function MF_IdentificacionValida ($Usuario, $Codigo)
  {
  $Consulta = "SELECT codigo FROM acceso WHERE usuario='$Usuario'";
  $CodigoCorrecto = BD_ConsultaValor ($Consulta);

  return ( password_verify ($Codigo, $CodigoCorrecto) );
  }

//--------------------------------------------------------------
// Prepara un mensaje de error
function MF_Error ($Texto)
  {
  $Respuesta = "<P><SPAN CLASS=\"error\">Error: </SPAN>$Texto\n";
  return $Respuesta;
  }

//--------------------------------------------------------------
// Prepara un mensaje de éxito
function MF_Exito ($Texto)
  {
  $Respuesta = "<P><SPAN CLASS=\"bien\">Éxito: </SPAN>$Texto\n";
  return $Respuesta;
  }

//--------------------------------------------------------------
// Comprueba un código y prepara un mensaje de error
function MF_CompruebaCodigo ($Codigo)
  {
  $Respuesta = "";
  $Texto = "la contraseña es demasiado corta (como mínimo debe " .
           "tener " . MF_MIN_PASSWORD_TEXTO . " caracteres)";

  if ( strlen ($Codigo) < MF_MIN_PASSWORD )
    { $Respuesta = MF_Error ($Texto); }

  return $Respuesta;
  }

//--------------------------------------------------------------
// Cambia el código corespondiente a un usuario
function MF_CambiaCodigo ($Usuario, $Codigo)
  {
  $Codigo = password_hash ($Codigo, PASSWORD_BCRYPT);
  $Consulta = "UPDATE acceso SET codigo='$Codigo' WHERE usuario='$Usuario'";
  return BD_EjecutaConsulta ($Consulta);
  }

?>
