#!/usr/bin/python -u
# coding: utf-8

#--------------------------------------------------------------------
# Fichero:  empareja.py
# Objetivo: Averiguar parejas que maximicen la felicidad
# Fecha:    D.10.11.2019
# Autor:    Pedro Reina
# Licencia: Dominio público
#           https://creativecommons.org/publicdomain/zero/1.0/deed.es
#--------------------------------------------------------------------

#------------------------------------------------
# Variables del programa
#------------------------------------------------

# El archivo con la base de datos
BaseDato = 'maxifeli.db'

# Cuántos linajes vamos a crear
TotalLinaje = 9

# Cuántas generaciones vamos a crear en cada linaje
TotalGeneracion = 15

# Cuántos hijos va a tener cada individuo
TotalHijos = 10

# Cuántos supervivientes vamos a dejar de cada generación
TotalSupervivientes = 100

# Archivos TTF con las fuentes
FuenteRegular = '/usr/share/fonts/truetype/droid/DroidSerif-Regular.ttf'
FuenteNegrita = '/usr/share/fonts/truetype/droid/DroidSerif-Bold.ttf'

# Tamaños de las letras
TamanoGrande = 16
TamanoNormal = 14

# Grosor de las líneas
Grosor = 0.5

# El PDF final
DocumentoPDF = 'parejas.pdf'

# El txt final
DocumentoTXT = 'parejas.txt'

# Los márgenes de las páginas, en centímetros
Margen = 1.5

# La distancia entre líneas de texto, en centímetros
Interlineado = 0.8

#------------------------------------------------
# Módulos necesarios
#------------------------------------------------

# Another Python SQLite Wrapper
import apsw

# Para averiguar la fecha y la hora
import datetime

# Para manejar aleatoriedad
import random

# Para escribir en Unicode en un archivo
import codecs

# Configuración de ReportLab
import reportlab.rl_config

# Creación de PDF con ReportLab
from reportlab.pdfgen import canvas
from reportlab.lib.units import cm
from reportlab.lib.pagesizes import A4
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import Table, TableStyle
from reportlab.lib import colors

#------------------------------------------------
# Variables globales
#------------------------------------------------

# La lista con los nombres y usuarios
ListaNombreUsuario = []

# La lista con los usuarios
ListaUsuario = []

# El diccionario con las preferencias
DiccionarioPreferencias = {}

# La mayor felicidad que se puede obtener con las preferencias
MayorFelicidadPosible = 0

#------------------------------------------------
# Funciones
#------------------------------------------------

#------------------------------------------------
# Dice la fecha y la hora
def FechaHora():
    Fecha = datetime.datetime.now().strftime('%Y-%m-%d')
    Hora = datetime.datetime.now().strftime('%H:%M:%S')
    return Fecha + u' ◊ ' + Hora

#------------------------------------------------
# Devuelve la lista de nombres y usuarios de la base de datos
def LeeNombreUsuario(Conexion):
    Lista = []
    Consulta = ('SELECT nombre, usuario FROM acceso')

    Cursor = Conexion.cursor()
    for Dato in Cursor.execute(Consulta):
        Lista.append((Dato[0], Dato[1]))

    return Lista

#------------------------------------------------
# Devuelve la lista de usuarios
def CreaListaUsuario():
    Lista = []

    for NombreUsuario in ListaNombreUsuario:
        Usuario = NombreUsuario[1]
        Lista.append(Usuario)

    return Lista

#------------------------------------------------
# Devuelve un diccionario de preferencias de la base de datos
def LeePreferencias(Conexion):
    Diccionario = {}

    # Por defecto no hay preferencias establecidas
    for NombreUsuario in ListaNombreUsuario:
        Activo = NombreUsuario[1]
        for NombreUsuarioPasivo in ListaNombreUsuario:
            Pasivo = NombreUsuarioPasivo[1]
            if Activo == Pasivo: Diccionario[(Activo, Pasivo)] = 0
            else:                Diccionario[(Activo, Pasivo)] = 0

    # Consultamos las preferencias que hay en la base de datos
    Consulta = ('SELECT activo, pasivo, cantidad FROM preferencia')
    Cursor = Conexion.cursor()
    for Dato in Cursor.execute(Consulta):
        Activo = Dato[0]
        Pasivo = Dato[1]
        Cantidad = Dato[2]
        Diccionario[(Activo, Pasivo)] = Cantidad

    # Devolvemos el diccionario creado
    return Diccionario

#------------------------------------------------
# Calcula la mayor felicidad que se podría alcanzar con estas preferencias
def CalculaMayorFelicidadPosible():
    Respuesta = 0

    for Activo in ListaUsuario:
        Preferencias = [DiccionarioPreferencias[(Activo, Pasivo)]
                        for Pasivo in ListaUsuario]
        Respuesta += max(Preferencias)

    return Respuesta

#------------------------------------------------
# Genera un pdf y un txt con las parejas y la felicidad
def GeneraDocumentos(Parejas, Felicidad):
    # Creamos el diccionario de nombres
    DiccionarioNombres = {}
    for NombreUsuario in ListaNombreUsuario:
        Nombre = NombreUsuario[0]
        Usuario = NombreUsuario[1]
        DiccionarioNombres[Usuario] = Nombre

    # Preparamos el pdf
    Pdf = canvas.Canvas(DocumentoPDF, pagesize=A4)
    Pdf.setAuthor('MaxiFeli')
    Pdf.setTitle('Maximizador de felicidad')

    # Preparamos el fichero de texto
    Salida = codecs.open(DocumentoTXT, 'w', encoding='UTF-8')
    Salida.write(Descripcion + '\n')

    # Guardamos las dimensiones de la página
    Anchura, Altura = A4

    # Escribimos el título
    Pdf.setFont('Regular', TamanoGrande)
    Linea = Altura - Margen
    Pdf.drawCentredString(Anchura/2, Linea, Descripcion)
    Linea -= Interlineado

    # Montamos una tabla: en cada fila un nombre y su pareja
    DatoTabla = []

    # La primera fila de la tabla es una cabecera
    Fila = []
    Fila.append('Nombre')
    Fila.append('Pareja')
    DatoTabla.append(Fila)

    # En las demás filas están los nombres
    for Primero in ListaUsuario:
        Segundo = Parejas[Primero]
        Fila = []
        NombrePrimero = DiccionarioNombres[Primero]
        NombreSegundo = DiccionarioNombres[Segundo]
        Fila.append(NombrePrimero)
        Fila.append(NombreSegundo)
        DatoTabla.append(Fila)
        # Escribimos en el fichero de texto
        Salida.write(NombrePrimero + '\t' + NombreSegundo + '\n')

    # El tamaño de la letra de la tabla debe permitir que quepa en anchura
    PosX = 551
    TamanoTabla = TamanoNormal
    while PosX > 550:
        Tabla = Table(DatoTabla)
        Tabla.setStyle(TableStyle([
              ('FONT', (0, 0), (1, 0), 'Negrita'),
              ('FONT', (0, 1), (-1, -1), 'Regular'),
              ('FONTSIZE', (0, 0), (-1, -1), TamanoTabla),
              ('ALIGN', (0,0), (-1, -1), 'LEFT'),
              ('VALIGN', (0,0), (-1, -1), 'MIDDLE'),
              ('LEFTPADDING', (0,0), (-1, -1), 4),
              ('RIGHTPADDING', (0,0), (-1, -1), 4),
              ('TOPPADDING', (0,0), (-1, -1), 2),
              ('BOTTOMPADDING', (0,0), (-1, -1), 6),
              ('BOX', (0,0), (-1,-1), Grosor, colors.black),
              ('GRID', (0,0), (-1,-1), Grosor, colors.black)
              ]))
        PosX, PosY = Tabla.wrapOn(Pdf, Anchura-2*Margen, Altura)
        TamanoTabla -= 0.2
    Tabla.drawOn(Pdf, Margen, Linea-PosY+0.2*cm)

    # Fin de página
    InformeFelicidad = ('Felicidad: ' + str(Felicidad) + ' / ' +
                        str(MayorFelicidadPosible))
    Pdf.drawString(Margen, Linea-PosY-Interlineado, InformeFelicidad)
    Pdf.showPage()

    # Escribimos el PDF
    Pdf.save()

    # Cerramos el archivo de texto
    Salida.write(InformeFelicidad + '\n')
    Salida.close()

#------------------------------------------------
# Calcula la felicidad de un emparejamiento
def CalculaFelicidad(Emparejamiento):
    Felicidad = 0
    for i, Primero in enumerate(Emparejamiento):
        if i % 2 == 0:
            Segundo = Emparejamiento[i+1]
            Felicidad += DiccionarioPreferencias[(Primero, Segundo)]
            Felicidad += DiccionarioPreferencias[(Segundo, Primero)]
    return Felicidad

#------------------------------------------------
# Averigua la felicidad máxima de una lista de candidatos y felicidades
def FelicidadMaxima(Lista):
    ListaFelicidades = [Elemento[1] for Elemento in Lista]
    return max(ListaFelicidades)

#------------------------------------------------
# Engendra un hijo a partir de un candidato
def Engendra(Candidato):
    # El hijo al principio es como el padre
    Hijo = list(Candidato)

    # Elegimos dos posiciones al azar
    i, j = random.sample(range(TotalUsuario), 2)

    # Intercambiamos los elementos de esas posiciones
    Hijo[i], Hijo[j] = Hijo[j], Hijo[i]

    # Devolvemos el hijo
    return Hijo

#------------------------------------------------
# Función auxiliar para ordenar por felicidad decreciente
def OrdenaAux(x): return -x[1]

#------------------------------------------------
# Busca un emparejamiento que maximice la felicidad
# usando un algoritmo genético con varios intentos
def BuscaEmparejamientoAG():
    # Informamos al usuario
    print 'Algoritmo genético:'

    # Guardamos los mejores de cada linaje
    MejoresPorLinaje = []

    # Generamos tantos linajes independientes como nos pidan
    for linaje in range(TotalLinaje):
        print '  Linaje', str(linaje+1) + ':',
        # Creamos la lista de emparejamientos candidatos
        # Cada candidato es una lista en la que se emparejan dos consecutivos
        ListaCandidatos = []

        # La generación cero de candidatos se crea al azar
        for i in range(TotalSupervivientes):
            Candidato = random.sample(ListaUsuario, TotalUsuario)
            Felicidad = CalculaFelicidad(Candidato)
            ListaCandidatos.append((Candidato, Felicidad))
        MaximaFelicidad = FelicidadMaxima(ListaCandidatos)
        print MaximaFelicidad,

        # Las demás generaciones se crean quedándonos los mejores candidatos
        for g in range(TotalGeneracion):
            # Cada superviviente tiene hijos
            for i in range(TotalSupervivientes):
                Padre = ListaCandidatos[i][0]
                for j in range(TotalHijos):
                    Hijo = Engendra(Padre)
                    Felicidad = CalculaFelicidad(Hijo)
                    ListaCandidatos.append((Hijo, Felicidad))
            # Ordenamos los candidatos por felicidad decreciente
            ListaCandidatos.sort(key=OrdenaAux)
            # Nos quedamos los mejores candidatos
            ListaCandidatos = ListaCandidatos[:TotalSupervivientes]
            # Vemos cuánta felicidad tenemos ahora
            MaximaFelicidad = ListaCandidatos[0][1]
            print MaximaFelicidad,
        print '=>', MaximaFelicidad
        # Guardamos el mejor individuo de este linaje
        MejoresPorLinaje.append(ListaCandidatos[0])

    # Ordenamos los mejores de cada linaje por felicidad decreciente
    MejoresPorLinaje.sort(key=OrdenaAux)

    # Devolvemos nuestro mejor individuo y su felicidad
    return MejoresPorLinaje[0][0], MejoresPorLinaje[0][1]

#------------------------------------------------
# Busca un emparejamiento que maximice la felicidad
# usando un algoritmo de optimización de parejas
def BuscaEmparejamientoOP():
    # Informamos al usuario
    print 'Algoritmo de optimización de parejas:',

    # Creamos una lista con la felicidad de cada posible pareja
    ListaParejaFelicidad = []
    for i, Primero in enumerate(ListaUsuario):
        for j, Segundo in enumerate(ListaUsuario):
            if i>j:
                Felicidad = (DiccionarioPreferencias[(Primero, Segundo)] +
                             DiccionarioPreferencias[(Segundo, Primero)])
                ListaParejaFelicidad.append(((Primero, Segundo),Felicidad))

    # Ordenamos las parejas por felicidad decreciente
    ListaParejaFelicidad.sort(key=OrdenaAux)

    # Creamos la lista de emparejamientos obtenidos
    # En la lista se emparejan dos usuarios consecutivos
    ListaObtenidos = []

    # Creamos la lista de los usuarios que aún no hemos emparejado
    ListaDisponibles = list(ListaUsuario)

    # Iremos sumando la felicidad que aporte cada pareja
    FelicidadObtenida = 0

    # Mientras queden usuarios por emparejar
    while len(ListaDisponibles):
        # Nos quedamos con la mejor pareja y la eliminamos
        Pareja, Felicidad = ListaParejaFelicidad.pop(0)
        # Sumamos la felicidad
        FelicidadObtenida += Felicidad
        # Vemos los usuarios que forman la pareja elegida
        Primero, Segundo = Pareja
        # Nos guardamos los usuarios
        ListaObtenidos.append(Primero)
        ListaObtenidos.append(Segundo)
        # Los usuarios ya no están disponibles
        ListaDisponibles.remove(Primero)
        ListaDisponibles.remove(Segundo)
        # Quitamos las parejas en las que haya alguno de estos usuarios
        # Atención al 'reversed': es necesario para no saltarse elementos
        for Elemento in reversed(ListaParejaFelicidad):
             if (Primero in Elemento[0]) or (Segundo in Elemento[0]):
                 ListaParejaFelicidad.remove(Elemento)
        # Informamos de la felicidad obtenida
        print Felicidad,
    print '=>', FelicidadObtenida

    # Devolvemos nuestra solución y su felicidad
    return ListaObtenidos, FelicidadObtenida

#------------------------------------------------
# Prepara las parejas de un emparejamiento
def PreparaParejas(Emparejamiento):
    Respuesta = {}

    for i, Primero in enumerate(Emparejamiento):
        if i%2 == 0:
            Segundo = Emparejamiento[i+1]
            Respuesta[Primero] = Segundo
            Respuesta[Segundo] = Primero

    return Respuesta

#------------------------------------------------
# El programa
#------------------------------------------------

# Informamos al usuario
print 'empareja'
print '========'
print

# Averiguamos la fecha y hora
Descripcion = FechaHora()

# Informamos al usuario
print 'Base de datos:', BaseDato
print 'Momento:', Descripcion

# Conversión de medidas: todas las hemos definido en cm
# pero se usan en puntos tipográficos
Margen *= cm
Interlineado *= cm

# Si las fuentes que cargamos no están completas, que no se queje ReportLab
reportlab.rl_config.warnOnMissingFontGlyphs = 0

# Cargamos las fuentes que vamos a usar
pdfmetrics.registerFont(TTFont ('Regular', FuenteRegular))
pdfmetrics.registerFont(TTFont ('Negrita', FuenteNegrita))

# Abrimos conexión a la base de datos
Conexion = apsw.Connection(BaseDato)

# Las listas de usuarios, nombres y preferencias
print 'Leyendo datos....',
ListaNombreUsuario = LeeNombreUsuario(Conexion)
DiccionarioPreferencias = LeePreferencias(Conexion)
ListaUsuario = CreaListaUsuario()
MayorFelicidadPosible = CalculaMayorFelicidadPosible()
print 'hecho.'

# Cerramos la conexión
Conexion.close(True)

# Comprobamos que los nombres son emparejables
TotalUsuario = len(ListaNombreUsuario)
if TotalUsuario % 2:
    print 'Hace falta un número par de nombres.'
else:
    # La mayor felicidad que podremos conseguir
    print 'La mayor felicidad posible con estos datos:', MayorFelicidadPosible

    # Buscamos el mejor emparejamiento posible de dos formas distintas
    MejorEncontradoAG, FelicidadAG = BuscaEmparejamientoAG()
    MejorEncontradoOP, FelicidadOP = BuscaEmparejamientoOP()

    # Nos quedamos con la mejor
    if FelicidadAG > FelicidadOP:
        MejorEncontrado = MejorEncontradoAG
        Felicidad = FelicidadAG
    else:
        MejorEncontrado = MejorEncontradoOP
        Felicidad = FelicidadOP

    # Informamos
    print 'Mayor felicidad encontrada:', Felicidad

    # Preparamos las parejas de cada usuario
    Parejas = PreparaParejas(MejorEncontrado)

    # Generamos el PDF y el txt
    print 'Archivo pdf:', DocumentoPDF
    print 'Archivo txt:', DocumentoTXT
    print 'Generando archivos...',
    GeneraDocumentos(Parejas, Felicidad)
    print 'hecho.'

# Hemos acabado; informamos al usuario
print 'Terminado.'
